/*
 * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 * 
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@

    Change History (most recent first):

 */
    
/* loclibrary.h                                                      
 * ----------------------------------------------------------------------
 * Header file for localization library                                       
 * Originally created by jsantamaria: 3 may 2004                         
 * ----------------------------------------------------------------------
 */
 
#ifndef _loclibrary_h_
#define _loclibrary_h_


#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

int PathForResourceW ( HMODULE module, const wchar_t *name, wchar_t *locFile, int locFileLen);
int PathForResourceWithPathW ( const wchar_t *path, const wchar_t *name, wchar_t *locFile, int locFileLen);

int PathForResourceA ( HMODULE module, const char *name, char *locFile, int locFileLen);
int PathForResourceWithPathA ( const char *path, const char *name, char *locFile, int locFileLen);


#ifdef UNICODE
#define PathForResource PathForResourceW
#define PathForResourceWithPath PathForResourceWithPathW
#else
#define PathForResource PathForResourceA
#define PathForResourceWithPath PathForResourceWithPathA
#endif // UNICODE


#ifdef __cplusplus
}
#endif // __cplusplus


#endif // _loclibrary_h_
