/*
 * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 * 
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@

    Change History (most recent first):

$Log: SharedSecret.h,v $
Revision 1.3  2005/04/06 02:04:49  shersche
<rdar://problem/4066485> Registering with shared secret doesn't work

Revision 1.2  2005/03/03 19:55:21  shersche
<rdar://problem/4034481> ControlPanel source code isn't saving CVS log info


*/

    
#pragma once

#include "resource.h"


//---------------------------------------------------------------------------------------------------------------------------
//	CSharedSecret
//---------------------------------------------------------------------------------------------------------------------------

class CSharedSecret : public CDialog
{
	DECLARE_DYNAMIC(CSharedSecret)

public:
	CSharedSecret(CWnd* pParent = NULL);   // standard constructor
	virtual ~CSharedSecret();

// Dialog Data
	enum { IDD = IDR_SECRET };

	void
	Commit( CString zone );

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()

public:

	CString m_key;
	CString m_secret;
};
