//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by ExplorerPluginRes.rc
//
#define IDS_NAME                        106
#define IDS_WEB_SITES                   107
#define IDS_PRINTERS                    109
#define IDS_MDNSRESPONDER_NOT_AVAILABLE 110
#define IDS_FIREWALL                    111
#define IDB_LOGO                        115
#define IDI_BUTTON_2K                   115
#define IDI_BUTTON_XP                   118
#define IDB_ABOUT                       119
#define IDC_COMPONENT                   1001
#define IDC_LEGAL                       1002
#define IDC_LOGIN_USERNAME_TEXT         1182
#define IDC_LOGIN_PASSWORD_TEXT         1183
#define ID_Menu                         40001

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        119
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1001
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
